/* Request.java */

/* 
 * Copyright (C) 1996-97 Mark Boyns <boyns@sdsu.edu>
 *
 * This file is part of Muffin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package muffin;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * @author Mark Boyns
 */
public class Request extends Message
{
    private String command = null;
    private String url = null;
    private String protocol = null;
    private byte[] data = null;
    
    public void read (InputStream in) throws IOException
    {
	statusLine = readLine (in);

	StringTokenizer st = new StringTokenizer (statusLine);
	command = (String) st.nextToken ();
	url = (String) st.nextToken ();
	protocol = (String) st.nextToken ();

	readHeaders (in);

	if (command.equals ("POST"))
	{
	    try
	    {
		int n = Integer.parseInt (getHeaderField ("Content-length"));
		data = new byte[n];
		in.read (data, 0, n);
	    }
	    catch (Exception e)
	    {
		System.out.println ("Malformed or missing POST Content-length");
	    }
	}
    }

    public void write (OutputStream out) throws IOException
    {
	String s = toString ();
	out.write (s.getBytes (), 0, s.length ());
	out.flush ();
    }

    public String toString ()
    {
	if (command.equals ("POST"))
	{
	    StringBuffer buf = new StringBuffer (super.toString ());
	    buf.append (getData ());
	    buf.append ("\r\n");
	    buf.append ("\r\n");
	    return buf.toString ();
	}
	else
	{
	    return super.toString ();
	}
    }

    public String getRequest ()
    {
	return statusLine;
    }

    public String getCommand ()
    {
	return command;
    }
    

    public String getURL ()
    {
	return url;
    }

    public void setURL (String url)
    {
	this.url = url;
    }

    public String getProtocol ()
    {
	return protocol;
    }

    public String getHost ()
    {
	String url = getURL ();
	String s = url.substring (7, url.indexOf ('/', 7));
	if (s.indexOf (':') != -1)
	{
	    return s.substring (0, s.indexOf (':'));
	}
	return s;
    }

    public int getPort ()
    {
	int port = 80;
	String url = getURL ();
	String s = url.substring (7, url.indexOf ('/', 7));
	if (s.indexOf (':') != -1)
	{
	    try
	    {
		port = Integer.parseInt (s.substring (s.indexOf (':') + 1));
	    }
	    catch (Exception e)
	    {
		System.out.println ("Invalid port in " + url);
	    }
	}
	return port;
    }

    public String getData ()
    {
	return new String (data);
    }

    public String getPath ()
    {	
	String str = getURL ();
	int pos = 0;
	for (int i = 0; i < 3; i++)
	{
	    pos = str.indexOf ('/', pos);
	    pos++;
	}
	pos--;
	return str.substring (pos);
    }
    
    public String getDocument ()
    {
	String path = getPath ();
	int n = path.lastIndexOf ('/');
	if (n == path.length () - 1)
	{
	    n = path.lastIndexOf ('/', n - 1);
	}
	if (n < 0)
	{
	    return "/";
	}
	else
	{
	    return path.substring (n + 1);
	}
    }
}
